﻿(* Raima.Rdm is the system module that contains the ADO.NET data provider
   for RDM *)
open Raima.Rdm

(* EXAMPLE - SQL06 for F# using ADO.NET *)
(* When you run this example, two databases are created each with a table.
 * Each table is populated with a few rows and finally the data read with
 * a single query.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection(String) Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *     Raima.Rdm.RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown exceptions
 * but good programming practices would dictate that this occur in
 * a full application.
 *)

let DropDatabase (conn : RdmConnection) name =
    let cmd = conn.CreateCommand()

    (* Attempt to drop this database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. *)
    try
        cmd.CommandText <- "DROP DATABASE " + name
        cmd.ExecuteNonQuery() |> ignore
    with
        | exn -> ()

let CreateThisDatabase (conn : RdmConnection) name cmds =
    let cmd = conn.CreateCommand()

    cmd.CommandText <- "CREATE DATABASE " + name
    cmd.ExecuteNonQuery() |> ignore

    cmds |>
        Array.iter(fun c ->
            cmd.CommandText <- c
            cmd.ExecuteNonQuery() |> ignore
        )

    cmd.CommandText <- "COMMIT"
    cmd.ExecuteNonQuery() |> ignore

    cmd.CommandText <- "CLOSE " + name
    cmd.ExecuteNonQuery() |> ignore

let CreateDatabases conn =
    (* Drop any existing copies of the databases. *)
    DropDatabase conn "SQL06_1"
    DropDatabase conn "SQL06_2"

    (* Attempt to create the databases *)
    CreateThisDatabase conn "SQL06_1" [| "CREATE TABLE Table1 (id INTEGER PRIMARY KEY, str CHAR(30))" |]
    CreateThisDatabase conn "SQL06_2" [| "CREATE TABLE Table2 (id INTEGER, str CHAR(30))" |]

(* Create the connection with a simple connection string *)
let conn = new RdmConnection("host=local;openmode=x")
conn.Open()

CreateDatabases conn

conn.ChangeDatabase("SQL06_1;SQL06_2")

(* Create a new command *)
let cmd = conn.CreateCommand()

(* Insert one row into the table in the first database *)
cmd.CommandText <- "INSERT INTO Table1 VALUES 1, 'database1 1'"
cmd.ExecuteNonQuery() |> ignore

(* Insert three rows into the table in the second database *)
[1..3] |>
    List.iter(fun ii ->
        cmd.CommandText <- "INSERT INTO Table2 VALUES 1, 'database2 " + (string ii) + "'"
        cmd.ExecuteNonQuery() |> ignore
    )

(* Scan through the data, generating the result *)
cmd.CommandText <- "SELECT Table1.str, Table2.str FROM Table1, Table2 WHERE Table1.id = Table2.id"
let rdr1 = cmd.ExecuteReader()

while (rdr1.Read()) do
    printfn "%s -- %s" (string (rdr1.Item(0))) (string (rdr1.Item(1)))

conn.Close()

printfn "Hit <Enter> to continue"
System.Console.ReadLine() |> ignore

